package compravenda

import (
	"strconv"
)

var mapaDeVendas *map[int]*Venda 

var idGenerator int = 0;
 
type Venda struct {
	id int
	meioDePgmto MeioDePagamento
	produto *Produto
	quantidade int
}

func NewVenda(meioDePagamento MeioDePagamento, produtoVendido *Produto, quantidadeVendida int) *Venda{
	idGenerator += 1
	
	return &Venda{idGenerator, meioDePagamento, produtoVendido, quantidadeVendida}
}

func (v *Venda) GetID() int{
	return v.id
}

func (v *Venda) String() string {
	return "id: " + strconv.FormatInt(int64(v.id),10) + " " + v.produto.descricao + " " +  strconv.FormatInt(int64(v.quantidade),10)
}

func GetVendas() *map[int]*Venda {
	return mapaDeVendas
}

func SetVendas (vendas *map[int]*Venda) {
	mapaDeVendas = vendas
}

//TODO: Adicionar Todos os produtos ao arquivo de vendas por produto

// Calcula a renda bruta e o lucro (bel) obtido nas vendas para cada produto
func VendaPorProduto() map[int]*BrutoELucro{
	
	// Aloca memoria para o mapa de valor bruto e lucro (bel) obtidos por produto
	bel := make(map[int]*BrutoELucro)
	
	vendas := *mapaDeVendas
	produtos := *mapaDeProdutos
	
	// Cria um mapa com todos os produtos e bels zerados
	for i,v := range produtos {
		p := produtos[i]
		bel[p.codigo] = &BrutoELucro{v,0,0}
	}
	
	// Itera sobre todas as vendas feitas
	for k,v := range vendas {
		
		precoProduto := vendas[k].produto.getPreco()
		lucroProduto := vendas[k].produto.getLucro()
		
		brutoVenda := float64(vendas[k].quantidade)*precoProduto
		lucroVenda := float64(vendas[k].quantidade)*lucroProduto
		
		chave := v.produto.codigo
		
		// Checa se esse produto ja esta presente no mapa de vendas brutas
		_ , ok := bel[chave]
		
		// Se nao, adiciona ele ao mapa com o valor bruto arrecadado na compra
		if !ok {
			bel[chave] = &BrutoELucro{item: v.produto, Bruto: brutoVenda, Lucro: lucroVenda}
		} else {
			// Caso sim, somente incrementa o valor bruto
			bel[chave].Bruto += brutoVenda
			bel[chave].Lucro += lucroVenda
		}
	}
	
	return bel
}

// Calcula a renda bruta e o lucro (bel) obtido nas vendas por meio de pagamento
func BelPorMeioDePagamento() map[MeioDePagamento]*BrutoELucro{
	
	// Aloca memoria para o mapa de vendas e valor bruto obtido
	bel := make(map[MeioDePagamento]*BrutoELucro)
	
	vendas := *mapaDeVendas
	
	meios := AllMeiosDePagamento()

	// Cria um mapa com todos os meios de pagamento e bels zerados
	for i,v := range meios {
		m := meios[i]
		bel[m] = &BrutoELucro{v,0,0}
	}
	
	// Itera sobre todas as vendas feitas
	for k := range vendas {
		
		meio := vendas[k].meioDePgmto
		
		precoProduto := vendas[k].produto.getPreco()
		lucroProduto := vendas[k].produto.getLucro()
		
		brutoVenda := float64(vendas[k].quantidade)*precoProduto
		lucroVenda := float64(vendas[k].quantidade)*lucroProduto
		
		bel[meio].Bruto += brutoVenda
		bel[meio].Lucro += lucroVenda
		
	}
	
	return bel
}
